import argparse as ap
import sys
from tweeds.query import Query
from tweeds.tweet_scrape import search


def banner():
    print("""	
        ████████╗░██╗░░░░░░░██╗███████╗███████╗██████╗░░██████╗
        ╚══██╔══╝░██║░░██╗░░██║██╔════╝██╔════╝██╔══██╗██╔════╝
        ░░░██║░░░░╚██╗████╗██╔╝█████╗░░█████╗░░██║░░██║╚█████╗░
        ░░░██║░░░░░████╔═████║░██╔══╝░░██╔══╝░░██║░░██║░╚═══██╗
        ░░░██║░░░░░╚██╔╝░╚██╔╝░███████╗███████╗██████╔╝██████╔╝
        ░░░╚═╝░░░░░░╚═╝░░░╚═╝░░╚══════╝╚══════╝╚═════╝░╚═════╝░

        by Achyuth Jois M
    """)


banner()


def config(args: ap.Namespace):
    c = Query()
    c.search = args.s
    c.username = args.u
    c.limit = args.limit
    c.json = args.json
    c.since = args.since
    c.until = args.until
    c.near = args.near
    c.minLikes = args.minLikes
    c.csv = args.csv
    c.minReplies = args.minReplies
    c.minRetweets = args.minRetweets
    c.silent = args.silent
    c.verified = args.verified
    c.geoCode = args.geocode
    c.links = args.links
    c.videos = args.videos
    c.images = args.images
    c.media = args.media
    c.year = args.year
    c.today = args.today

    return c


def process_args(args: ap.Namespace):
    q = config(args)
    search(q)


def main():
    parse = ap.ArgumentParser(description="Scrape twitter user tweets")

    parse.add_argument("-u", help="User's Tweets you want to scrape.")
    parse.add_argument(
        "-s", help="Search for Tweets containing this word or phrase.")
    parse.add_argument(
        '--since', type=str, help="Filter Tweets sent since date (Example: \"2017-12-27 20:30:15\" or 2017-12-27)."
    )
    parse.add_argument(
        '--until', type=str, help="Filter Tweets sent until date (Example: \"2017-12-27 20:30:15\" or 2017-12-27)."
    )
    parse.add_argument(
        '--limit', type=int, help="Number of Tweets to pull"
    )
    parse.add_argument(
        '--near', type=str, help="Find tweets near a particular location"
    )
    parse.add_argument(
        '--geocode', type=str, help="Search for geocoded Tweets."
    )
    parse.add_argument(
        '--year', type=int, help="Filter Tweets before specified year."
    )
    parse.add_argument(
        '--today', help="Filter Tweets from today", action='store_true'
    )
    parse.add_argument(
        '--verified', help="Display Tweets only from verified users (Use with -s).", action='store_true'
    )
    parse.add_argument(
        '--links', help="Exclude tweets containing one or more links.", action='store_true'
    )
    parse.add_argument(
        "--videos", help="Display only Tweets with videos.", action='store_true'
    )
    parse.add_argument(
        "--images", help="Display only Tweets with images.", action='store_true'
    )
    parse.add_argument(
        "--media", help="Display Tweets with only images or videos.", action='store_true'
    )

    parse.add_argument(
        '--minLikes', type=int, help="Minimun likes for the tweet"
    )
    parse.add_argument(
        '--minRetweets', type=int, help="Minimun retweets for the tweet"
    )
    parse.add_argument(
        '--minReplies', type=int, help="Minimun replies for the tweet"
    )
    parse.add_argument(
        '--json', type=str, help="File to write the JSON output to.")
    parse.add_argument(
        '--csv', type=str, help="To store the output in CSV"
    )
    parse.add_argument(
        '--silent', help="Don't print the tweets(Only works while taking an output!)[Type anything]", action='store_true'
    )

    args = parse.parse_args(args=None if sys.argv[1:] else ['--help'])

    process_args(args)



import dataclasses
from typing import Optional


@dataclasses.dataclass
class Query:
    """
        Class used to query in twitter
    """
    username: Optional[str] = None
    search: Optional[str] = None
    since: Optional[str] = None
    until: Optional[str] = None
    json: Optional[str] = None
    limit: Optional[int] = None
    near: Optional[str] = None
    minLikes: Optional[int] = None
    csv: Optional[str] = None
    minReplies: Optional[int] = None
    minRetweets: Optional[int] = None
    silent: bool = False
    verified: bool = False
    geoCode: Optional[str] = None
    links: bool = False
    videos: bool = False
    images: bool = False
    media: bool = False
    year: Optional[int] = None
    today: bool = False

    def __init__(self) -> None:
        pass



import snscrape.modules.twitter as api
import pandas as pd
import json
from tweeds.query import Query
from datetime import date


def make_json(data, jsonFilePath):
    with open(jsonFilePath, 'w', encoding='utf-8') as jsonf:
        jsonf.write(json.dumps(data, indent=4))


def printRes(tweet: api.Tweet):
    print(f"{tweet.id} {tweet.date} <{tweet.user.username}> {tweet.rawContent} \n")


def toOBJ(tweet: api.Tweet) -> object:
    return {
        "id": tweet.id,
        "date": tweet.date.strftime('%Y/%m/%d'),
        "username": tweet.user.username,
        "content": tweet.rawContent,
        "likes": tweet.likeCount,
        "retweet": tweet.retweetCount,
        "reply": tweet.replyCount,
        "user": {
            "username": tweet.user.username,
            "followers": tweet.user.followersCount,
            "displayName": tweet.user.displayname,
            "id": tweet.user.id
        },
        "url": tweet.url
    }


def search(q: Query) -> None:
    """Print tweets"""
    query = ""

    if q.search:
        query += f"{q.search} "
    if q.username:
        query += f"(from:{q.username}) "
    if q.today:
        today = date.today().strftime("%Y-%m-%d")
        query += f"since:{today} "
    if q.year:
        query += f"since:{q.year}-01-01 "
    if q.until:
        query += f"until:{q.until} "
    if q.since:
        query += f"since:{q.since} "
    if q.minLikes:
        query += f"min_faves:{q.minLikes} "
    if q.minReplies:
        query += f" min_replies:{q.minReplies} "
    if q.minRetweets:
        query += f" min_retweets:{q.minRetweets} "
    if q.near:
        query += f"near:{q.near} "
    if q.geoCode:
        query += f"geocode:{q.geoCode} "
    if q.verified:
        query += f"filter:verified "
    if q.media:
        query += f"filter:media "
    if q.videos and not q.media and not q.images:
        query += f"filter:native_video "
    if q.images and not q.media and not q.videos:
        query += f"filter:images "
    if q.links:
        query += "-filter:links "
    jsonObj = {}
    csvObj = []

    for i, tweet in enumerate(api.TwitterSearchScraper(query).get_items()):
        if q.limit:
            if i == q.limit:
                break
        jsonObj[tweet.id] = toOBJ(tweet)
        csvObj.append(
            [tweet.id, tweet.date, tweet.rawContent, tweet.url,
             tweet.likeCount, tweet.retweetCount, tweet.replyCount, tweet.sourceLabel[12:]]
        )
        if q.silent:
            if q.csv or q.json:
                pass
        else:
            printRes(tweet)

    if q.json:
        if q.json.find(".json") != -1:
            make_json(jsonObj, q.json)
            print("Output saved in JSON!")

    if q.csv:
        df = pd.DataFrame(
            csvObj, columns=["ID", "Date", "Tweet", "URL", "Likes", "Retweet", "Replies", "Source"])
        df.to_csv(q.csv)
        print("Output saved in CSV!")



from setuptools import setup, find_packages

long_description = """"Scrape tweets from any Twitter user profile. Twitter API alternative to scrape Twitter hashtags, threads, images, videos, statistics,
and Twitter history. Export data in JSON and CSV format. This project enables you to extract large amounts of data from Twitter.
It lets you do much more than the Twitter API, because it doesn't have rate limits and you don't even need to have a **Twitter account, a registered app,
or Twitter API key.**"""

requirements = [x.strip() for x in open("requirements.txt", "r").readlines()]

setup(
    name='tweeds',
    version='2.1.2.1',
    author='Achyuth Jois M',
    author_email='developer.arjm@gmail.com',
    packages=find_packages(include=['tweeds', 'tweeds.*']),
    install_requires=requirements,
    license='MIT',
    license_files=('LICENSE.md'),
    url='https://github.com/achyuthjoism/Twitter-OSINT',
    description='An advanced Twitter scraping & OSINT tool.',
    long_description=long_description,
    entry_points={
        'console_scripts': [
            'tweeds = tweeds.main:main',
        ]
    },
)
